// 230428

module StrobeGenerator #(
	parameter	CLOCK_HZ	= 10_000_000,
	parameter	PERIOD_US	= 100
)(
	input		Clock,
	input		Reset,
	output reg	Strobe_o
);
	
	localparam DELAY = (CLOCK_HZ / 1_000_000) * PERIOD_US - 1;
	localparam WIDTH = $clog2(DELAY + 1);
	
	initial begin
		if(DELAY <= 0)
			$fatal(0, "Wrong DELAY value: %d", DELAY);
	end
	
	reg [WIDTH-1:0] Counter_r;
	
	always @(posedge Clock, negedge Reset) begin
		if(!Reset) begin
			Counter_r <= DELAY;
			Strobe_o  <= 1'b0;
		end else if(!Counter_r) begin
			Counter_r <= DELAY;
			Strobe_o  <= 1'b1;
		end else begin
			Counter_r <= Counter_r - 1'b1;
			Strobe_o  <= 1'b0;
		end 
	end
	
endmodule
