
`default_nettype none

module top(
	input wire Reset,
	output wire [3:0] ComPWM_o,
	output wire [7:0] SegPWM_o
);

	// Generator sygnau zegarowego
	parameter CLOCK_HZ = 14_000_000;
	wire Clock;
	OSCH #(
		.NOM_FREQ("14.00")
	) OSCH_inst(
		.STDBY(1'b0),
		.OSC(Clock),
		.SEDSTDBY()
	);
	
	// Testowy licznik
	wire CountEnable_w;
	StrobeGenerator #(
		.CLOCK_HZ(CLOCK_HZ),
		.PERIOD_US(100_000)
	) CountEnable_m(
		.Clock(Clock),
		.Reset(Reset),
		.Strobe_o(CountEnable_w)
	);
	
	reg [15:0] Counter_r;
	always @(posedge Clock, negedge Reset) begin
		if(!Reset)
			Counter_r <= 0;
		else if(CountEnable_w)
			Counter_r <= Counter_r + 1'b1;
	end
	
	// Dekodery wywietlaczy
	wire [6:0] Segments0_w;
	Decoder7seg Decoder0(
		.Enable_i(1'b1),
		.Data_i(Counter_r[3:0]),
		.Segments_o(Segments0_w)
	);
	
	wire [6:0] Segments1_w;
	Decoder7seg Decoder1(
		.Enable_i(1'b1),
		.Data_i(Counter_r[7:4]),
		.Segments_o(Segments1_w)
	);
	
	wire [6:0] Segments2_w;
	Decoder7seg Decoder2(
		.Enable_i(1'b1),
		.Data_i(Counter_r[11:8]),
		.Segments_o(Segments2_w)
	);
	
	wire [6:0] Segments3_w;
	Decoder7seg Decoder3(
		.Enable_i(1'b1),
		.Data_i(Counter_r[15:12]),
		.Segments_o(Segments3_w)
	);
	
	
	// Instancja sterownika wywietlacza LCD
	LCD #(
		.CLOCK_HZ(CLOCK_HZ),
		.CHANGE_COM_US(5000)
	) LCD_inst(
		.Clock(Clock),
		.Reset(Reset),
		/*
		.iDigit3(8'b10000000),
		.iDigit2(8'b00000000),
		.iDigit1(8'b00000000),
		.iDigit0(8'b00000000),
		*/
		.Digit3_i({1'b0, Segments3_w}),
		.Digit2_i({1'b0, Segments2_w}),
		.Digit1_i({1'b0, Segments1_w}),
		.Digit0_i({1'b0, Segments0_w}),
		
		.ComPWM_o(ComPWM_o),
		.SegPWM_o(SegPWM_o)
	);
	

endmodule

`default_nettype wire
